/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.51 from the
 * contents of Devel.xs. Do not edit this file, edit Devel.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "Devel.xs"
/* $Id: Devel.xs 20 2011-10-11 02:05:01Z jo $
 *
 * This is free software, you may use it and distribute it under the same terms as
 * Perl itself.
 *
 * Copyright 2011 Joachim Zobel
 *
 * This module gives external access to the functions needed to create
 * and use XML::LibXML::Nodes from C functions. These functions are made
 * accessible from Perl to have cleaner dependencies.
 * The idea is to pass xmlNode * pointers (as typemapped void *) to and
 * from Perl and call the functions that turns them to and from
 * XML::LibXML::Nodes there.
 *
 * Be aware that using this module gives you the ability to easily create
 * segfaults and memory leaks.
 */

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"


#include <stdlib.h>

/* XML::LibXML stuff */
#include <libxml/xmlmemory.h>
#include "perl-libxml-mm.h"

#undef NDEBUG
#include <assert.h>

/* libxml2's custom memory-allocator and tracking API was removed in 2.14.
 * Apple SDKs (macOS 15.4 / iOS 18.4 / etc.) pre-flag it as deprecated even
 * while shipping libxml2 < 2.14. Only expose debug_memory() and mem_used()
 * when the API is genuinely supported by the installed libxml2. */
#if LIBXML_VERSION < 21400 && !defined(LIBXML_HAS_DEPRECATED_MEMORY_ALLOCATION_FUNCTIONS)
#define HAVE_LIBXML_MEMORY_DEBUG 1
#endif

#ifdef HAVE_LIBXML_MEMORY_DEBUG
static void *	xmlMemMallocAtomic(size_t size)
{
    return xmlMallocAtomicLoc(size, "none", 0);
}

static int debug_memory()
{
    return xmlGcMemSetup( xmlMemFree,
                          xmlMemMalloc,
                          xmlMemMallocAtomic,
                          xmlMemRealloc,
                          xmlMemStrdup);
}
#endif

#line 67 "Devel.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

#line 211 "Devel.c"

XS_EUPXS(XS_XML__LibXML__Devel_node_to_perl); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_node_to_perl)
{
    dVAR; dXSARGS;
    if (items < 1 || items > 2)
       croak_xs_usage(cv,  "n, o = NULL");
    {
	void *	n = INT2PTR(void *,SvIV(ST(0)))
;
	void *	o;
#line 75 "Devel.xs"
        xmlNode *node = n;
        xmlNode *owner = o;
#line 226 "Devel.c"
	SV *	RETVAL;

	if (items < 2)
	    o = NULL;
	else {
	    o = INT2PTR(void *,SvIV(ST(1)))
;
	}
#line 78 "Devel.xs"
        RETVAL = PmmNodeToSv(node , owner?owner->_private:NULL );
#line 237 "Devel.c"
	RETVAL = sv_2mortal(RETVAL);
	ST(0) = RETVAL;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_XML__LibXML__Devel_node_from_perl); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_node_from_perl)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "sv");
    {
	SV *	sv = ST(0)
;
#line 86 "Devel.xs"
        xmlNode *n = PmmSvNodeExt(sv, 0);
#line 256 "Devel.c"
	void *	RETVAL;
	dXSTARG;
#line 88 "Devel.xs"
        RETVAL = n;
#line 261 "Devel.c"
	XSprePUSH;
	PUSHi(PTR2IV(RETVAL));
    }
    XSRETURN(1);
}


XS_EUPXS(XS_XML__LibXML__Devel_refcnt_inc); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_refcnt_inc)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "n");
    {
	void *	n = INT2PTR(void *,SvIV(ST(0)))
;
#line 96 "Devel.xs"
        xmlNode *node = n;
#line 280 "Devel.c"
#line 98 "Devel.xs"
        PmmREFCNT_inc(((ProxyNode *)(node->_private)));
#line 283 "Devel.c"
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_XML__LibXML__Devel_refcnt_dec); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_refcnt_dec)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "n");
    {
	void *	n = INT2PTR(void *,SvIV(ST(0)))
;
#line 104 "Devel.xs"
        xmlNode *node = n;
#line 300 "Devel.c"
	int	RETVAL;
	dXSTARG;
#line 106 "Devel.xs"
        RETVAL = PmmREFCNT_dec(((ProxyNode *)(node->_private)));
#line 305 "Devel.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_XML__LibXML__Devel_refcnt); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_refcnt)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "n");
    {
	void *	n = INT2PTR(void *,SvIV(ST(0)))
;
#line 114 "Devel.xs"
        xmlNode *node = n;
#line 324 "Devel.c"
	int	RETVAL;
	dXSTARG;
#line 116 "Devel.xs"
        RETVAL = PmmREFCNT(((ProxyNode *)(node->_private)));
#line 329 "Devel.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}


XS_EUPXS(XS_XML__LibXML__Devel_fix_owner); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_fix_owner)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "n, p");
    {
	void *	n = INT2PTR(void *,SvIV(ST(0)))
;
	void *	p = INT2PTR(void *,SvIV(ST(1)))
;
#line 125 "Devel.xs"
        xmlNode *node = n;
        xmlNode *parent = p;
#line 351 "Devel.c"
	int	RETVAL;
	dXSTARG;
#line 128 "Devel.xs"
        RETVAL = PmmFixOwner(node->_private , parent->_private);
#line 356 "Devel.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}

#ifdef HAVE_LIBXML_MEMORY_DEBUG
#define XSubPPtmpAAAA 1


XS_EUPXS(XS_XML__LibXML__Devel_mem_used); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_XML__LibXML__Devel_mem_used)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;
#line 137 "Devel.xs"
        RETVAL = xmlMemUsed();
#line 378 "Devel.c"
	XSprePUSH;
	PUSHi((IV)RETVAL);
    }
    XSRETURN(1);
}

#endif
#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_XML__LibXML__Devel); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_XML__LibXML__Devel)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        newXS_deffile("XML::LibXML::Devel::node_to_perl", XS_XML__LibXML__Devel_node_to_perl);
        newXS_deffile("XML::LibXML::Devel::node_from_perl", XS_XML__LibXML__Devel_node_from_perl);
        newXS_deffile("XML::LibXML::Devel::refcnt_inc", XS_XML__LibXML__Devel_refcnt_inc);
        newXS_deffile("XML::LibXML::Devel::refcnt_dec", XS_XML__LibXML__Devel_refcnt_dec);
        newXS_deffile("XML::LibXML::Devel::refcnt", XS_XML__LibXML__Devel_refcnt);
        newXS_deffile("XML::LibXML::Devel::fix_owner", XS_XML__LibXML__Devel_fix_owner);
#if XSubPPtmpAAAA
        newXS_deffile("XML::LibXML::Devel::mem_used", XS_XML__LibXML__Devel_mem_used);
#endif

    /* Initialisation Section */

#line 62 "Devel.xs"
#ifdef HAVE_LIBXML_MEMORY_DEBUG
    if (getenv("DEBUG_MEMORY")) {
        debug_memory();
    }
#endif

#if XSubPPtmpAAAA
#endif
#line 435 "Devel.c"

    /* End of Initialisation Section */

#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
